#ifndef BITMAP_H
#define BITMAP_H

struct PCXHEADER
	{
	BYTE ID;
	BYTE version;
	BYTE encoding;
	BYTE bpp;
	WORD xmin;
	WORD ymin;
	WORD xmax;
	WORD ymax;
	WORD hres;
	WORD vres;
	BYTE colorMap[48];
	BYTE reserved;
	BYTE nplanes;
	WORD bytesPerLine;
	WORD palType;
	BYTE filler[58];
	};

class Bitmap
	{
	public:

	Bitmap(void);
	~Bitmap(void);

	int addColor(RGBTRIPLE);
	BYTE copyTile(BYTE *,int,int);
	void corner(void);
	BYTE createBlank(int,int);
	void createFromDC(HDC,int,int);
	void drawToDC(HDC,Bitmap* = NULL,int=0,int=0);
	void drawToLargeDC(BYTE,HDC,int,int,Bitmap* =NULL,int=0,int=0);
	void flipImage(int,bool=false);
	RGBTRIPLE getRGBPixel(int,int);
	BYTE loadFile(char*,int=256,int=1024,int=1024);
	BYTE saveFile(char*,int=0,int=0);
	BYTE numColors(void);
	BYTE setColor(BYTE,RGBTRIPLE,bool check=false);
	BYTE setRGBPixel(int,int,RGBTRIPLE);
	void swapPal(int,int);

	BYTE saveBMP(char *filename,int,int);
	BYTE savePCX(char *filename,int,int);
	BYTE savePNG(char *filename,int,int);
	
	//Inline functions

	inline RGBTRIPLE getColor(BYTE c)
		{ return palette[c]; }
	inline int getHeight(void)
		{ return height; }
	inline BYTE getPixel(int x,int y) 
		{ if((y >= height) || (x >= width)) return 0; return pixels[y * width + x]; }
	inline int getWidth(void)
		{ return width; }
	inline void setPixel(int x,int y,BYTE c)
		{ if((y >= height) || (x >= width)) return; pixels[width * y + x] = c; }

	private:

	int convert256(RGBTRIPLE,int,int);

	BYTE loadBMP(char*,int,int,int);
	BYTE loadPCX(char*,int,int,int);
	BYTE loadPNG(char*,int,int,int);

	BYTE load1BitBMP(int);
	BYTE load4BitBMP(int);
	BYTE load8BitBMP(int);
	BYTE load16BitBMP(int);
	BYTE load24BitBMP(int);
	BYTE load32BitBMP(int);

	BYTE load1BitPCX(int,int);
	BYTE load4BitPCX(int,int);
	BYTE load8BitPCX(int,int);
	BYTE load24BitPCX(int,int);

	BYTE loadPalPNG(int,FILE*,int);
	BYTE loadRGBPNG(int,FILE*);
	
	void setTransparency();

	int					width;
	int					height;
	
	int					colorsUsed;
	
	BYTE*				pixels;
	RGBTRIPLE			palette[256];

	HBITMAP				BMP;
	};

#define RGBaddress(x,y) ((x) + ((y) * (width)))

//BMP header and info
extern BITMAPFILEHEADER	bmfh;
extern BITMAPINFOHEADER	bmih;
extern RGBQUAD			*aColors;
extern BYTE				*aBitmapBits;

//PCX header and info
extern PCXHEADER		pcxh;
extern RGBTRIPLE		tempPal[256];


#endif

