#include "stdafx.h"

#include "changeback.h"
#include "ff3se.h"
#include "resource.h"
#include "combatback.h"
#include "options.h"

int yScroll = 0;

HDC backDC[60];
HBITMAP backBMP[60];

bool backsLoaded = false;

BYTE oldBack;

/*************************************/
void changeBackCleanup(void)
	{
	int i;

	for(i = 0;i < 60;i++)
		{
		DeleteDC(backDC[i]);
		DeleteObject(backBMP[i]);
		}
	}

/*************************************/
//Displays the Change Background dialog
void changeBackground(void)
	{
	HWND hScroll;
		
	//Save original background image for if Cancel is clicked on
	oldBack = options.getBackImage();

	//Create and display the Change Background dialog
	hDlg = CreateDialog(hInst,MAKEINTRESOURCE(IDD_CHANGE_BACK),hWnd,changeBackHook);
	
	BringWindowToTop(hDlg);
	ShowWindow(hDlg,SW_NORMAL);
	EnableWindow(hWnd,false);

	hScroll = GetDlgItem(hDlg,IDC_CHANGE_BACK_SCROLL);
	SetScrollRange(hScroll,SB_CTL,0,13,false);
	
	yScroll = (options.getBackImage() / 4) * 4;
	if(yScroll > 52)
		yScroll = 52;
	SetScrollPos(hScroll,SB_CTL,yScroll / 4,false);

	drawBackgrounds();
	}

/*************************************/
//Draws the backgrounds to select from
//Only draws the 8 that are on screen
void drawBackgrounds(void)
	{
	int i;
	HDC picDC;
	HWND hPic;
	int j;

	for(i = 0;i < 8;i++)
		{
		hPic = GetDlgItem(hDlg,IDC_PIC_BACK1 + i);
		picDC = GetDC(hPic);
		BitBlt(picDC,0,0,128,128,backDC[i + yScroll],0,0,SRCCOPY);
		
		//Draw border around selected image
		if((i + yScroll) == options.getBackImage())
			{
			for(j = 0;j < 128;j++)
				{
				SetPixel(picDC,j,0,0xFFFFFF);
				SetPixel(picDC,0,j,0xFFFFFF);
				SetPixel(picDC,j,127,0xFFFFFF);
				SetPixel(picDC,127,j,0xFFFFFF);
				}
			}
		ReleaseDC(hPic,picDC);
		}
	}


/*************************************/
//Handles a mouse click
//Changes the selected image if an image is clicked on
void handleChangeBackMouseClick(int x,int y)
	{
	int i;
	HWND hPic;
	RECT r;

	//Convert x and y to screen coordinates

	GetWindowRect(hDlg,&r);

	x += r.left;
	y += r.top;
	
	for(i = 0;i < 8;i++)
		{
		hPic = GetDlgItem(hDlg,IDC_PIC_BACK1 + i);
		GetWindowRect(hPic,&r);
//		r.bottom = r.top + 128;			//Make sure image size is 128x128
//		r.right = r.left + 128;
		if(inRect(x,y,r))				//if the mouse was clicked on this pic
			{
			options.setBackImage(i + yScroll);
			drawBackgrounds();
			}
		}
	}

/*************************************/
//Handles the scroll bar to select which backgrounds to display
void handleChangeBackScroll(int val,HWND hScroll,int pos)
	{
	switch(val)
		{		
		case SB_LINEUP:
			pos = GetScrollPos(hScroll,SB_CTL) - 1;
			break;
		case SB_LINEDOWN:
			pos = GetScrollPos(hScroll,SB_CTL) + 1;
			break;
		case SB_PAGEUP:
			pos = GetScrollPos(hScroll,SB_CTL) - 2;
			break;
		case SB_PAGEDOWN:
			pos = GetScrollPos(hScroll,SB_CTL) + 2;
			break;
		case SB_ENDSCROLL:
			return;
			break;
		}
	
	if(pos < 0)
		pos = 0;
	if(pos > 13)
		pos = 13;
	SetScrollPos(hScroll,SB_CTL,pos,true);
			
	yScroll = pos * 4;;

	drawBackgrounds();
	}

/*************************************/
//Creates and loads all the backgrounds into DCs for be drawn quickly
void loadBackgrounds(void)
	{
	int i;
	HBITMAP tempBMP;
	Bitmap tempBmp;

	//Create default DCs 
	for(i = 0;i < 60;i++)
		{
		backDC[i] = CreateCompatibleDC(screenHdc);
		backBMP[i] = CreateCompatibleBitmap(screenHdc,128,128);
		SelectObject(backDC[i],backBMP[i]);
		}

	//Load 5 basic backgrounds from the resources
	for(i = 0;i < 5;i++)
		{
		DeleteObject(tempBMP);
		tempBMP = (HBITMAP)LoadImage(hInst,MAKEINTRESOURCE(IDB_BACK1 + i),IMAGE_BITMAP,0,0,LR_DEFAULTCOLOR);		
		SelectObject(backDC[i],tempBMP);
		}

	//Create 2 additional one color backgrounds
	tempBmp.createBlank(128,128);		//Create Black background
	tempBmp.drawToDC(backDC[5]);
	tempBmp.setColor(0,makeRGBTRIPLE(30,60,240));		//Create Blue background
	tempBmp.drawToDC(backDC[6]);

	//Create 55 combat backgrounds
	for(i = 0;i < 49;i++)
		{
		combatBack[i].pic.drawToLargeDC(1,backDC[i + 7],64,12);
		}
	//Skip backgrounds 49 and 50
	for(i = 51;i < 55;i++)
		{
		combatBack[i].pic.drawToLargeDC(1,backDC[i + 5],64,12);
		}

	DeleteObject(tempBMP);
	backsLoaded = true;
	}

/*************************************/
//Creates the new backBMP and back2BMP based on the selected image
void setBackground(void)
	{
	HWND hLoad,hText;
	int x,y;
	RECT r;

	//Load the backgrounds if they haven't been loaded already
	if(!backsLoaded)
		{
		//Display loading Dialog while backgrounds load
		hLoad = CreateDialog(hInst,MAKEINTRESOURCE(IDD_LOADING),hWnd,NULL);
		SetWindowText(hLoad,ff3seTitle);	
		hText = GetDlgItem(hLoad,IDC_TEXT_LOAD);
		SetWindowText(hText,"Loading backgrounds ...");	
		GetWindowRect(hWnd,&r);
		SetWindowPos(hLoad,HWND_NOTOPMOST,r.left + 150,r.top + 100,0,0,SWP_NOSIZE | SWP_SHOWWINDOW);
		UpdateWindow(hLoad);
			
		//Load combat backgrounds
		loadCombatBacks(ROMname);
		
		//Load backgrounds and create BITMAPs and DCs
		loadBackgrounds();	
		
		DestroyWindow(hLoad);
		backsLoaded = true;
		}

	backBmp.createFromDC(backDC[options.getBackImage()],128,128);
	
	back2Bmp.createBlank(256,256);

	for(y = 0;y < 256;y++)
		{
		for(x = 0;x < 256;x++)
			{
			back2Bmp.setRGBPixel(x,y,backBmp.getRGBPixel(x / 2,y / 2));
			}
		}
	}

/*************************************/
BOOL CALLBACK changeBackHook(HWND hdlg,UINT message,WPARAM wParam,LPARAM lParam)
	{
	switch(message)
		{
		case WM_CLOSE:
			EnableWindow(hWnd,true);
			DestroyWindow(hDlg);
			return true;
			break;
		case WM_COMMAND:
			switch(LOWORD(wParam))
				{
				case IDC_CHANGE_BACK_OK:
					setBackground();
					EnableWindow(hWnd,true);
					DestroyWindow(hDlg);
					return true;
					break;					
				case IDC_CHANGE_BACK_CANCEL:
					options.setBackImage(oldBack);
					EnableWindow(hWnd,true);
					DestroyWindow(hDlg);
					return true;
					break;
				}
			break;
		case WM_LBUTTONDOWN:
			handleChangeBackMouseClick(LOWORD(lParam),HIWORD(lParam));
			return true;
			break;
		case WM_PAINT:
			drawBackgrounds();
			break;
		case WM_VSCROLL:
			handleChangeBackScroll(LOWORD(wParam),(HWND)lParam,HIWORD(wParam));
			break;			
		}

	return false;
	}
