#include "stdafx.h"

#include "character.h"
#include "ff3se.h"
#include "tile.h"

Character character[numCharacters];
RGBTRIPLE charPals[32][16];

BYTE *charTiles;
USHORT partsOffset[numPics * 6];

/*************************************/
Character::Character(void)
	{
	tiles = NULL;
	minor = false;
	fixedPalette = false;
	}

/*************************************/
Character::~Character(void)
	{
	SAFE_DELETE(tiles);
	}

/*************************************/
//Changes the Portrait's palette and updates the bitmap
void Character::changePalette(BYTE p)
	{
	int i,j;
	
	if(fixedPalette)
		return;

	palette = p;

	for(i = 0;i < 40;i++)
		{
		for(j = 0;j < 16;j++)
			{
			pics[i].setColor(j,charPals[palette][j]);
			}
		}
	}

/*************************************/
//Clears all sprite sfor this character
void Character::clear(void)
	{
	int i,j;
	int num = numPics;

	if(minor)
		num = numMinorPics;

	for(i = 0;i < num;i++)
		{
		pics[i].createBlank(16,24);
		for(j = 0;j < 16;j++)
			{
			pics[i].setColor(j,charPals[palette][j]);
			}
		}
	}

/*************************************/
//Copies a sprite from another character
//If copyAll is true copies all sprites
void Character::copyImage(int num,int copyAll)
	{
	int i;

	if(copyAll == -1)
		{
		memcpy(tiles,character[num].tiles,(181 * 64));
		palette = character[num].palette;
		}
	else
		{
		for(i = 0;i < 6;i++)
			{
			memcpy(tiles + (partsOffset[copyAll * 6 + i] * 2),
				character[num].tiles + (partsOffset[copyAll * 6 + i] * 2),64);
			}
		}
	
	createBitmaps();
	}


/*************************************/
//Creates all the Bitmaps for the Map Sprite
void Character::createBitmaps(void)
	{
	int i,j;
	int x,y;

	if(fixedPalette)		//Set up palette for Esper Terra
		{
		for(i = 0;i < 40;i++)
			{
			pics[i].createBlank(16,24);
			pics[i].setColor(0,makeRGBTRIPLE(31 * 8,5 * 8,0));
			pics[i].setColor(1,makeRGBTRIPLE(11 * 8,2 * 8,8 * 8));
			pics[i].setColor(2,makeRGBTRIPLE(31 * 8,31 * 8,27 * 8));
			pics[i].setColor(3,makeRGBTRIPLE(18 * 8,0,0));
			pics[i].setColor(4,makeRGBTRIPLE(31 * 8,31 * 8,31 * 8));
			pics[i].setColor(5,makeRGBTRIPLE(25 * 8,16 * 8,31 * 8));
			pics[i].setColor(6,makeRGBTRIPLE(31 * 8,31 * 8,31 * 8));
			pics[i].setColor(7,makeRGBTRIPLE(17 * 8,14 * 8,17 * 8));
			pics[i].setColor(8,makeRGBTRIPLE(23 * 8,18 * 8,0));
			pics[i].setColor(9,makeRGBTRIPLE(18 * 8,0,0));
			pics[i].setColor(10,makeRGBTRIPLE(25 * 8,16 * 8,31 * 8));
			pics[i].setColor(11,makeRGBTRIPLE(16 * 8,11 * 8,27 * 8));
			pics[i].setColor(12,makeRGBTRIPLE(20 * 8,5 * 8,4 * 8));
			pics[i].setColor(13,makeRGBTRIPLE(30 * 8,16 * 8,10 * 8));
			pics[i].setColor(14,makeRGBTRIPLE(14 * 8,7 * 8,0));
			pics[i].setColor(15,makeRGBTRIPLE(18 * 8,15 * 8,10 * 8));
			}
		}
	else
		{
		for(i = 0;i < 40;i++)
			{
			pics[i].createBlank(16,24);
			for(j = 0;j < 16;j++)
				{
				pics[i].setColor(j,charPals[palette][j]);
				}
			}
		}

	for(i = 0;i < 40;i++)
		{
		for(y = 0;y < 3;y++)
			{
			for(x = 0;x < 2;x++)
				{
				pics[i].copyTile(tiles + partsOffset[i * 6 + y * 2 + x] * 2,x,y);
				}
			}
		}	
	}

/*************************************/
void Character::flipImage(int flip)
	{
	int i;
	int num = numPics;

	if(minor)
		num = numMinorPics;
	
	for(i = 0;i < num;i++)
		{
		pics[i].flipImage(flip);
		}
	}

/*************************************/
//Loads the graphics for this map sprite
void Character::load(int handle,BYTE num)
	{
	int address;
	BYTE c;
	BYTE *ptr;
	int i;

	if(num >= 22)
		minor = true;
	else
		minor = false;
		
	//Get palette number
	lseek(handle,0x2D02B + num,SEEK_SET);
	read(handle,&palette,1);
	if(palette > 8)
		palette = 0;

	if(num == 18)			//Fix the palette for the Esper Terra
		fixedPalette = true;
	else
		fixedPalette = false;

	//Load tiles
	lseek(handle,0x00D2F2 + num * 2,SEEK_SET);
	read(handle,&c,1);
	address = c;
	read(handle,&c,1);
	address += c * 256;	

	lseek(handle,0x00D43C + num * 2,SEEK_SET);
	read(handle,&c,1);

	address += c * 65536;
	address = mem2ROM(address);
	address -= 0x150200;
	
	tileOffset = address;

	ptr = charTiles + address;
	
	tiles = (BYTE *)malloc(181 * 64);
		
	for(i = 0;i < 181;i++)
		{
		loadTile(ptr,&(tiles[i * 64]));
		ptr += 32;
		}
	
	createBitmaps();
	}

/*************************************/
//Saves the tileset and palette number used by character to the ROM
void Character::save(int handle,BYTE num)
	{
	int i,x,y;
	bool tilesDone[181];
	int offset;
	USHORT *ptr;
	BYTE *saveTiles;

	lseek(handle,0x2D02B + num,SEEK_SET);

	write(handle,&palette,1);

	saveTiles = (BYTE *)malloc(181 * 32);

	ZeroMemory(saveTiles,181 * 32);
	
	for(i = 0;i < 181;i++)
		tilesDone[i] = false;

	ptr = partsOffset;

	for(i = 0;i < 40;i++)
		{
		for(y = 0;y < 3;y++)
			{
			for(x = 0;x < 2;x++)
				{
				offset = *(ptr++);
				if(!tilesDone[offset >> 5])
					{
					createTile((pics[i]),x,y,(saveTiles + offset));
					tilesDone[offset >> 5] = true;
					}
				}
			}
		}
	
	lseek(handle,0x150200 + tileOffset,SEEK_SET);
	for(i = 0;i < 181;i++)
		{
		if(tilesDone[i])
			write(handle,saveTiles + i * 32,32);
		else
			lseek(handle,32,SEEK_CUR);
		}

	SAFE_DELETE(saveTiles);
	}

/*************************************/
//Creates the character's sprites from the Bitmap passed in
void Character::setImage(Bitmap& bmp)
	{
	int i,j;
	bool found;
	int x,y;
	int x2,y2;
	int sprite;
	RGBTRIPLE color;

	SAFE_DELETE(tiles);
	tiles = (BYTE *)malloc(181 * 64);

	for(i = 0;i < 181;i++)
		{
		found = false;
		j = 0;
		do {
			if(partsOffset[j++] == (i * 32))
				{
				found = true;
				}
		} while((!found) && (j < 240));
		if(found == false)
			continue;
		j--;

		sprite = (j / 6);
		x2 = (sprite % 8) * 16;
		x2 += (j % 2) * 8;
		y2 = (sprite / 8) * 24;
		y2 += ((j - (j / 6) * 6) / 2) * 8;

		for(y = 0;y < 8;y++)
			{
			for(x = 0;x < 8;x++)
				{
				tiles[i * 64 + x + y * 8] = bmp.getPixel(x + x2,y + y2);
				}
			}
		}
	
	for(i = 0;i < 16;i++)
		{
		color = bmp.getColor(i);
		charPals[palette][i].rgbtRed = color.rgbtRed;
		charPals[palette][i].rgbtGreen = color.rgbtGreen;
		charPals[palette][i].rgbtBlue = color.rgbtBlue;
		}
	
	createBitmaps();
	}

/*************************************/
//Sets pixcel x,y in the ith sprite for the character
//Updates the coresponding tile
void Character::setPixel(BYTE num,int x,int y,BYTE c)
	{
	tiles[(partsOffset[num * 6 + (y / 8) * 2 + (x / 8)] * 2) + (y % 8) * 8 + (x % 8)] = c;

	createBitmaps();
	}

/*************************************/
//Makes color # num transparent by swapping it with color 0
void Character::setTransparency(BYTE num)
	{
	RGBTRIPLE temp;
	int i;

	int n = numPics;

	if(fixedPalette)
		return;

	if(minor)
		n = numMinorPics;

	for(i = 0;i < n;i++)
		pics[i].swapPal(0,num);

	temp = charPals[palette][0];

	charPals[palette][0].rgbtBlue = charPals[palette][num].rgbtBlue;
	charPals[palette][0].rgbtGreen = charPals[palette][num].rgbtGreen;
	charPals[palette][0].rgbtRed = charPals[palette][num].rgbtRed;
	charPals[palette][num].rgbtBlue = temp.rgbtBlue;
	charPals[palette][num].rgbtGreen = temp.rgbtGreen;
	charPals[palette][num].rgbtRed = temp.rgbtRed;
	}

/*************************************/
//Updates the Bitmap with the current palette info
void Character::updatePalette(void)
	{
	int i,j;

	if(fixedPalette)
		return;

	for(i = 0;i < 40;i++)
		{
		for(j = 0;j < 16;j++)
			{
			pics[i].setColor(j,charPals[palette][j]);
			}
		}
	}

/*************************************/
//Loads the sprites for all characters for the ROM
//filename is the name of the ROM to load from
bool loadCharacters(char *filename)
	{
	int handle;
	int i;

	handle = open(filename,O_RDWR | O_BINARY);

	if(handle == -1)
		return false;
	
	lseek(handle,0x0D03A,SEEK_SET);
	read(handle,partsOffset,numPics * 12);

	loadCharPals(handle);	
	loadCharTiles(handle);

	for(i = 0;i < numCharacters;i++)
		{
		character[i].load(handle,i);
		}

	close(handle);
	
	SAFE_DELETE(charTiles);

	return true;
	}

/*************************************/
//Loads all 32 palettes used for character sprites
void loadCharPals(int handle)
	{
	int i,j;
	BYTE *data;
	BYTE *ptr;
	BYTE c1,c2;

	data = (BYTE *)malloc(1024);

	lseek(handle,0x268200,SEEK_SET);

	read(handle,data,1024);

	ptr = data;

	for(i = 0;i < 32;i++)
		{
		for(j = 0;j < 16;j++)
			{
			c1 = *(ptr++);
			c2 = *(ptr++);		
			charPals[i][j] = getRGBTRIPLE(c1,c2);
			}
		}

	SAFE_DELETE(data);
	}

/*************************************/
//Loads the tiles used for character sprites
void loadCharTiles(int handle)
	{
	charTiles = (BYTE *)malloc(0x35000);

	lseek(handle,0x150200,SEEK_SET);

	read(handle,charTiles,0x35000);
	}


/*************************************/
//Saves all the character sprites to the ROM
bool saveCharacters(char *filename)
	{
	int handle;
	int i;

	handle = open(filename,O_RDWR | O_BINARY);

	if(handle == -1)
		{
		MessageBox(NULL,"Error saving character sprites data",ff3seTitle,MB_OK | MB_ICONWARNING | MB_TASKMODAL);
		return false;
		}
	
	saveCharPals(handle);
	
	for(i = 0;i < numCharacters;i++)
		{
		character[i].save(handle,i);
		}

	close(handle);

	return true;
	}

/*************************************/
//Saves all 32 Character sprite palettes to the ROM
void saveCharPals(int handle)
	{
	int i,j;
	BYTE c1,c2;

	lseek(handle,0x268200,SEEK_SET);

	for(i = 0;i < 32;i++)
		{
		for(j = 0;j < 16;j++)
			{
			getIntPal(charPals[i][j],&c1,&c2);
			write(handle,&c1,1);		
			write(handle,&c2,1);		
			}
		}

	lseek(handle,0x2D6500,SEEK_SET);

	for(i = 0;i < 6;i++)
		{
		for(j = 0;j < 16;j++)
			{
			getIntPal(charPals[i][j],&c1,&c2);
			write(handle,&c1,1);		
			write(handle,&c2,1);		
			}
		}

	}

