#include "stdafx.h"

#include "combatback.h"
#include "ff3se.h"
#include "tile.h"

Background combatBack[55];
RGBTRIPLE backPals[384][16];

Background::~Background(void)
	{
	SAFE_DELETE(tileSet);
	}

Background::Background(void)
	{
	tileSet = NULL;
	}

/*************************************/
//Creates the background's pic from the tileset, palette, and layout
void Background::createBitmap(void)
	{
	int x,y;
	int i;
	BYTE tile[64];

	pic.createBlank(256,152);
	
	for(i = 0;i < 48;i++)
		{
		pic.setColor(i,backPals[palette][i]);
		}

	//Only create the bitmap for the portion shown in the background image
	for(y = 1;y < 18;y++)
		{
		for(x = 8;x < 24;x++)
			{
			memcpy((BYTE *)tile,&(tileSet[layout[x][y].tile * 64]),64);
			if(layout[x][y].flipHorizontal)
				flipTileHorizontal(tile);
			if(layout[x][y].flipVertical)
				flipTileVertical(tile);
			for(i = 0;i < 64;i++)
				tile[i] += layout[x][y].palette;
			pic.copyTile((BYTE*)(&tile),x,y);
			}
		}
	
	//Once Bitmap is created. no longer need tileset in memory
	SAFE_DELETE(tileSet);
	}

/*************************************/
//Loads the background from the ROM
void Background::load(int handle,BYTE num)
	{
	char c;
	
	lseek(handle,0x270200 + num * 6 + 5,SEEK_SET);
	read(handle,&c,1);
	c &= 0x7f;
	
	palette = c * 3;

	loadTiles(handle,num);
	loadLayout(handle,num);
	createBitmap();
	}

/*************************************/
//Loads the tile layout
void Background::loadLayout(int handle,BYTE backNum)
	{
	BYTE c,d;
	int address;
	int x,y;
	BYTE *data;
	int t;
	int p;
	bool h,v;
	int val;

	data = (BYTE *)malloc(0x2000);

	lseek(handle,0x270200 + backNum * 6 + 3,SEEK_SET);
	read(handle,&c,1);

	c *= 2;

	lseek(handle,0x271A48 + c,SEEK_SET);
	read(handle,&d,1);
	address = d;
	read(handle,&d,1);
	address += d * 256;
	address += 0x270200;

	lseek(handle,address,SEEK_SET);

	decompress(handle,data);
	
	for(y = 0; y < 19;y++)
		{
		for(x = 0;x < 32;x++)
			{
			val = (y * 32 + x) * 2;
			
			t = data[val];
			if((data[val + 1] & 0x01) == 0x01)
				t += 64;

			if((data[val + 1] & 0x02) == 0x02)
				t += 256;

			p = ((data[val + 1]) & 0x3C) >> 2;
			p -= 5;
			p *= 16;
			h = false;
			v = false;

			if((data[val + 1] & 0x40) == 0x40)
				h = true;
			 if((data[val + 1] & 0x80) == 0x80)
				v = true;

			layout[x][y].flipHorizontal = h;
			layout[x][y].flipVertical = v;
			layout[x][y].palette = p;
			layout[x][y].tile = t;
			}
		}
	
	SAFE_DELETE(data);
	}

/*************************************/
//Loads all the tiles for the background
void Background::loadTiles(int handle,BYTE backNum)
	{
	int i;
	unsigned char c;
	int address1,address2;
	int num;
	BYTE *dataPtr;
	BYTE *tilePtr;
	int t;
	BYTE *data;
	int a;

	SAFE_DELETE(tileSet);
	tileSet = (BYTE *)malloc(36864);

	data = (BYTE *)malloc(0x2000);

	ZeroMemory(tileSet,36864);

	for(a = 0;a < 3;a++)
		{
		if(a == 0)
			t = 0;
		if(a == 1)
			t = 128;
		if(a == 2)
			t = 448;

		lseek(handle,0x270200 + backNum * 6 + a,SEEK_SET);
		
		read(handle,&c,1);
		if(c == 0xFF)
			continue;

		if((c & 0x80) > 0)
			num = 0x2000;
		else
			num = 0x1000;
	
		num /= 0x20;

		address1 = 0x271850 + (c & 0x7f) * 3;
		lseek(handle,address1,SEEK_SET);
		read(handle,&c,1);
		address2 = c;
		read(handle,&c,1);
		address2 += c * 256;
		read(handle,&c,1);
		address2 += c * 65536;

		address2 = mem2ROM(address2);
		lseek(handle,address2,SEEK_SET);
	
		if(c >= 0xE7)
			decompress(handle,data);
		else
			read(handle,data,num * 0x20);

		dataPtr = data;
		tilePtr = &(tileSet[t * 64]);

		for(i = 0;i < num;i++)
			{
			loadTile(dataPtr,tilePtr);
			dataPtr += 32;
			tilePtr += 64;
			}
		}

	SAFE_DELETE(data);
	}

/*************************************/
//Loads all 55 combat backgrounds from the ROM
bool loadCombatBacks(char *filename)
	{
	int handle;
	int i;

	handle = open(filename,O_RDWR | O_BINARY);

	if(handle == -1)
		return false;

	loadBackPals(handle);
	
	for(i = 0;i < 55;i++)
		{
		combatBack[i].load(handle,i);
		}

	close(handle);

	return true;
	}

/*************************************/
//Loads all the background palettes
void loadBackPals(int handle)
	{
	int	i;
	BYTE c,d;
	RGBTRIPLE *ptr;

	lseek(handle,0x270350,SEEK_SET);
	
	ptr = (RGBTRIPLE *)backPals;

	for(i = 0;i < 6144;i++)
		{
		read(handle,&c,1);
		read(handle,&d,1);
		
		*(ptr++) = getRGBTRIPLE(c,d);
		}
	}

