#include "stdafx.h"

#include "monsters.h"
#include "ff3se.h"

Monsters monsters;

/*************************************/
Monsters::Monsters(void)
	{
	maxTileSize = 226240;
	maxMoldSize = 2780;

	numMonsImages = -1;
	}
	
/*************************************/
//Deallocates memory used by Monsters
void Monsters::cleanup(void)
	{
	int i;

	SAFE_DELETE(molds8);
	SAFE_DELETE(molds16);

	for(i = 0;i < 450;i++)
		SAFE_DELETE(image[i].graphicsData);	
	}

/*************************************/
//Calculates the amount of space the images need in the ROM
long Monsters::calcROMSize(void)
	{
	int i;
	long size = 0;

	for(i = 0;i < numMonsImages;i++)
		if(image[i].used)
			size += image[i].ROMsize;
		
	return size;	
	}

/*************************************/
//Removese unused Images and Mold data
void Monsters::compress(void)
	{
	int i;

	for(i = 0;i < numMonsImages;i++)
		{
		if(!imageUsed(i))
			{
			removeImage(i);
			}
		}

	for(i = 0;i < numMolds8;i++)
		{
		if(!moldUsed(i,0))
			{
			removeMold(i,0);
			i--;
			}
		}
	for(i = 0;i < numMolds16;i++)
		{
		if(!moldUsed(i,1))
			{
			removeMold(i,1);	
			i--;				
			}
		}
	}

/*************************************/
//Returns the amount of free room for monsters in the ROM
int Monsters::getFreeRoom(void)
	{
	return maxTileSize - calcROMSize();
	}

/*************************************/
//Returns Monster # num
Monster* Monsters::getMonster(int num)
	{
	return &(monster[num]);
	}

/*************************************/
//Returns true if image #num is used by atleast one monster
bool Monsters::imageUsed(int num)
	{
	int i;

	for(i = 0;i < 415;i++)
		{
		if(monster[i].getImageNum() == num)
			return true;
		}

	return false;
	}

/*************************************/
//Loads the graphics for all 415 monsters and convert them to Bitmaps
void Monsters::loadMonsGraphics(int handle)
	{
	BYTE *ptr;
	int i;

	BYTE specs[415][5];
	
	lseek(handle,0x127200,SEEK_SET);

	read(handle,specs,415 * 5);

	ptr = (BYTE *)&specs;

	for(i = 0;i < 415;i++)
		{
		monster[i].loadStats(ptr);
		ptr += 5;
		}

	lseek(handle,0x297200,SEEK_SET);

	graphics = (BYTE *)malloc(0x49000);
	read(handle,graphics,0x39000);

	for(i = 0;i < 415;i++)
		{
		monster[i].loadGraphics();
		}

	}

/*************************************/
//Loads all the mold configurations for the monsters
void Monsters::loadMonsMolds(int handle)
	{
	USHORT num;
	int address1,address2;
	BYTE *ptr,*ptr2;
	BYTE c;
	int i,y,k;
	int address3 = 0x12B500;
		
	lseek(handle,0x12AA20,SEEK_SET);

	read(handle,&num,2);
	address1 = mem2ROM(num + 0xD20000);
	read(handle,&num,2);
	address2 = mem2ROM(num + 0xD20000);

	moldData = (BYTE *)malloc(maxMoldSize);
	lseek(handle,address1,SEEK_SET);
	read(handle,moldData,address3 - address1 + 1);
	
	ptr = moldData;

	numMolds8 = (address2 - address1 + 1) / 8;
	
	numMolds16 = (address3 - address2 + 1) / 32;
	
	SAFE_DELETE(molds8);
	SAFE_DELETE(molds16);
	
	molds8 = (BYTE *)malloc(numMolds8 * 64);	
	molds16 = (BYTE *)malloc(numMolds16 * 256);	

	ptr2 = molds8;
	for(i = 0;i < numMolds8;i++)
		{
		for(y = 0;y < 8;y++)
			{
			c = *(ptr++);

			*(ptr2++) = (c & 0x80) >> 7;
			*(ptr2++) = (c & 0x40) >> 6;
			*(ptr2++) = (c & 0x20) >> 5;
			*(ptr2++) = (c & 0x10) >> 4;
			*(ptr2++) = (c & 0x08) >> 3;
			*(ptr2++) = (c & 0x04) >> 2;
			*(ptr2++) = (c & 0x02) >> 1;
			*(ptr2++) = (c & 0x01) >> 0;
			}
		}

	ptr2 = molds16;

	for(i = 0;i < numMolds16;i++)
		{
		for(y = 0;y < 16;y++)
			{
			for(k = 0;k < 2;k++)
				{
				c = *(ptr++);

				*(ptr2++) = (c & 0x80) >> 7;
				*(ptr2++) = (c & 0x40) >> 6;
				*(ptr2++) = (c & 0x20) >> 5;
				*(ptr2++) = (c & 0x10) >> 4;
				*(ptr2++) = (c & 0x08) >> 3;
				*(ptr2++) = (c & 0x04) >> 2;
				*(ptr2++) = (c & 0x02) >> 1;
				*(ptr2++) = (c & 0x01) >> 0;
				}
			}
		}

	}

/*************************************/
//Loads the names of all 415 monsters
void Monsters::loadMonsNames(int handle)
	{
	BYTE *temp;
	BYTE *ptr;
	char name[21];
	int i,j;

	temp = (BYTE *)malloc(384 * 10);

	lseek(handle,0xFC250,SEEK_SET);

	read(handle,temp,384 * 10);

	ptr = temp;
	
	name[10] = 0;

	for(i = 0;i < 384;i++)
		{
		ZeroMemory(name,21);
		memcpy(name,ptr,10);		
		for(j = 0;j < 10;j++)
			{
			name[j] = tileChar(name[j]);
			}	
		for(j = 9;(j >= 0) && (name[j] == ' ');j--)
			{
			name[j] = 0;
			}
		monster[i].setName(name);
		ptr += 10;
		}
	
	//Read in Esper names
	lseek(handle,0x26F8E1,SEEK_SET);
	for(i = 384;i < 411;i++)
		{
		ZeroMemory(name,21);
		read(handle,temp,8);
		for(j = 0;j < 8;j++)
			{
			name[j] = tileChar(temp[j]);
			}
		for(j = 7;(j >= 0) && (name[j] == ' ');j--)
			{
			name[j] = 0;
			}
		monster[i].setName(name);
		}

	SAFE_DELETE(temp);
	}

/*************************************/
//Loads all 768 monster palettes
void Monsters::loadMonsPals(int handle)	
	{
	BYTE c1,c2;
	int i,j;
	BYTE *temp;
	BYTE *ptr;

	temp = (BYTE *)malloc(12288);

	lseek(handle,0x127A20,SEEK_SET);

	read(handle,temp,12288);

	ptr = temp;
	
	for(i = 0;i < 768;i++)
		{
		for(j = 0;j < 8;j++)
			{
			c1 = *(ptr++);
			c2 = *(ptr++);
		
			monsPals[i][j] = getRGBTRIPLE(c1,c2);
			
			}
		}
	
	SAFE_DELETE(temp);
	}

/*************************************/
//Loads graphics for all 415 monsters from the ROM passed as filename
//returns true on success, false if an error occured
bool Monsters::loadMonsters(char *filename)
	{
	int handle;
    int i;
    
	handle = open(filename,O_BINARY | O_RDWR);
	if(handle == -1)
		return false;
	
	numMonsImages = -1;

	for(i = 0;i < 450;i++)
		{
		image[i].graphicsData = NULL;
		image[i].used = true;
		image[i].offset = -1;		//Mark all offsets as empty
		}
	image[0].offset = 0;

	loadMonsPals(handle);
	loadMonsNames(handle);
	loadMonsMolds(handle);

	loadMonsGraphics(handle);

	close(handle);

	numMonsImages++;					//Account for image numbers starting at 0

	numMolds16 = -1;					//Count actual number of molds16

	for(i = 0;i < 415;i++)
		{
		if(monster[i].getSize() == 1)
			if(monster[i].getMoldNum() > numMolds16)
				numMolds16 = monster[i].getMoldNum();
		}
	
	numMolds16++;
	
	//Make sure all monsters use most efficient mold
	for(i = 0;i < 415;i++)
		{
		monster[i].getNewMold();
		monster[i].createImage();
		}
	
	return true;
	}

/*************************************/
//Compares all bytes of mold1 and mold2
//Size determines if it's a mold16 or mold8
//Returns true if they are identical molds
bool moldcmp(BYTE *mold1,BYTE *mold2,BYTE size)
	{
	int len;
	int x,y;

	len = (size == 0)?8:16;

	for(y = 0;y < len;y++)
		for(x = 0;x < len;x++)
			if(*(mold1++) != *(mold2++))
				return false;
	
	return true;
	}
 

/*************************************/
//Delete image # num and shifts the other images down accordingly
void Monsters::removeImage(int num)
	{
	image[num].used = false;
	}

/*************************************/
//Deletes mold #num and shifts the other molds down accordingly
void Monsters::removeMold(int num,BYTE size)
	{
	int i;

	if(size == 0)
		{
		numMolds8--;
		for(i = num;i < numMolds8;i++)
			{
			memcpy(&molds8[i * 64],&molds8[(i + 1) * 64],64);
			}
		molds8 = (BYTE *)realloc(molds8,(numMolds8) * 64);
		}
	else
		{
		numMolds16--;
		for(i = num;i < numMolds16;i++)
			{
			memcpy(&molds16[i * 256],&molds16[(i + 1) * 256],256);
			}
		molds16 = (BYTE *)realloc(molds16,(numMolds16) * 256);
		}

	for(i = 0;i < 415;i++)
		{
		if((monster[i].getMoldNum() > num) && (monster[i].getSize() == size))
			monster[i].decreaseMold();
		}
	
	}

/*************************************/
//Saves the mold data to the ROM
void Monsters::saveMolds(int handle)
	{
	USHORT address;
	BYTE *ptr;
	int i,y,j;
	BYTE val;

	lseek(handle,0x12AA20,SEEK_SET);

	address = (SHORT)0xA824;			//Write offset of molds8 data
	
	write(handle,&address,2);

	lseek(handle,0x12AA24,SEEK_SET);

	//Write molds8 data
	ptr = molds8;
	for(i = 0;i < numMolds8;i++)
		{
		for(y = 0;y < 8;y++)
			{
			val = 0;
	
			for(j = 0;j < 8;j++)
				{
				if(*(ptr++))
					val += BIT(7 - j);
				}
			write(handle,&val,1);
			}
		}
	
	address = tell(handle) - 0x120200;	//Determine offset of mold16 data
	lseek(handle,0x12AA22,SEEK_SET);	
	write(handle,&address,2);
	lseek(handle,0x120200 + address,SEEK_SET);

	//Write molds16 data
	ptr = molds16;
	for(i = 0;i < numMolds16;i++)
		{
		for(y = 0;y < 16;y++)
			{
			val = 0;

			for(j = 0;j < 8;j++)
				if(*(ptr++))
					val += BIT(7 - j);
			write(handle,&val,1);

			val = 0;

			for(j = 0;j < 8;j++)
				if(*(ptr++))
					val += BIT(7 - j);
			write(handle,&val,1);
			}
		}
	}
	
/*************************************/
//Saves the monster data to the ROM
bool Monsters::saveMonsters(char *filename)
	{
	int handle;
	int moldsize;
	
	if(calcROMSize() > maxTileSize)
		{
		MessageBox(NULL,"Not enough room in ROM to save the monster data. You must delete one or more monster images before saving.",ff3seTitle,MB_OK | MB_ICONWARNING);
		return false;
		}

	//Try to make sure images display correctly in ff3usme
	if((numMolds8 < 128) && (numMolds16 < 54))
		numMolds8 = 128;

	moldsize = numMolds8 * 8 + numMolds16 * 32;

	if(moldsize > maxMoldSize)
		{
		MessageBox(NULL,"Too much mold data",ff3seTitle,MB_OK | MB_ICONWARNING);
		return false;
		}

	handle = open(filename,O_BINARY | O_RDWR);
	if(handle == -1)
		{
		MessageBox(NULL,"Error saving monster data",ff3seTitle,MB_OK | MB_ICONWARNING);
		return false;
		}

	saveTiles(handle);
	saveMolds(handle);
	savePals(handle);
	saveSpecs(handle);

	close(handle);
	
	return true;
	}
	
/*************************************/
//Saves the palette data to the ROM
void Monsters::savePals(int handle)
	{
	int i,j;
	BYTE c1,c2;

	lseek(handle,0x127A20,SEEK_SET);

	for(i = 0;i < 768;i++)
		{
		for(j = 0;j < 8;j++)
			{
//			c1 = monsPals[i][j].rgbtRed / 8;
//			c1 += ((monsPals[i][j].rgbtGreen / 8) & 0x07) << 5;
//			c2 = ((monsPals[i][j].rgbtGreen / 8) & 0x18) >> 3;
//			c2 += (monsPals[i][j].rgbtBlue / 8) << 2;
			
			getIntPal(monsPals[i][j],&c1,&c2);

			write(handle,&c1,1);
			write(handle,&c2,1);
			}
		}
	}

/*************************************/
//Saves the monster specs data to the ROM
void Monsters::saveSpecs(int handle)
	{
	int i;

	lseek(handle,0x127200,SEEK_SET);

	for(i = 0;i < 415;i++)
		{
		monster[i].saveSpecs(handle);
		}
	}

/*************************************/
//Saves the tile data to the ROM
void Monsters::saveTiles(int handle)
	{
	lseek(handle,0x297200,SEEK_SET);
	
	for(int i = 0;i < numMonsImages;i++)
		{
		if(image[i].used)
			{
			image[i].offset = (tell(handle) - 0x297200) / 8;
			write(handle,image[i].graphicsData,image[i].ROMsize);
			}
		}
	}

/*************************************/
//Updates all the monster Bitmaps from their image data
void Monsters::updateMonsters(void)
	{
	int i;

	for(i = 0;i < 411;i++)
		{
		monster[i].createBitmap();
		}
	}

/*************************************/
//Returns true if any Monster uses the mold passed in, false otherwise
bool Monsters::moldUsed(int num,BYTE size)
	{
	bool used = false;
	int i;

	for(i = 0;(i < numMonsImages) && (used == false);i++)
		{
		if((image[i].mold == num) && (image[i].size == size) &&(image[i].used))
			used = true;
		}

	return used;
	}

/*************************************/
//Calculates the number of tiles in a mold
int numTiles(BYTE *ptr,BYTE size)
	{
	int num = 0;
	int x,y;	

	for(y = 0;y < size;y++)
		for(x = 0;x < size;x++)
			num += *(ptr++);
	
	return num;
	}

