#include "stdafx.h"

#include "options.h"
#include "ff3se.h"
#include "resource.h"
#include "bitmap.h"
#include "tab1.h"
#include "tab3.h"

Options options;

Options::Options(void)
	{
	readConfig();
	}

/*************************************/
//Checks to see if the filename already exists in the recent file list,
//and adds it if the name doesn';t exist
void Options::addRecentFile(char *filename)
	{
	int i,j;

	for(i = 0;i < numRecentFiles;i++)
		{
		if(!strcmp(filename,recentFile[i]))		
			{
			for(j = i;j < numRecentFiles;j++)
				strcpy(recentFile[j],recentFile[j + 1]);
			numRecentFiles--;
			}
		}

	strcpy(recentFile[3],recentFile[2]);
	strcpy(recentFile[2],recentFile[1]);
	strcpy(recentFile[1],recentFile[0]);

	if(numRecentFiles != 4)
		numRecentFiles++;

	strcpy(recentFile[0],filename);

	}

/*************************************/
//Sets filename to the filename portion of the numth recent file name
//sets filename to NULL if there aren't i recent file names
void Options::getRecentFileName(BYTE num,char *filename)
	{
	UINT i = 0;
	int j;
	bool done = false;

	do {
		j = 0;
		while((i < strlen(recentFile[num])) && (recentFile[num][i] != '\\'))
			{
			filename[j++] = recentFile[num][i];
			i++;
			}
		i++;
		if(i > strlen(recentFile[num]))
			done = true;
	} while(!done);
	
	filename[j] = 0;
	}

/*************************************/
//Reads the config file to determine options
//If no file is found, loads default settings
void Options::readConfig(void)
	{
	HKEY softwareKey;
	HKEY companyKey;
	HKEY productKey;
	HKEY settingsKey;
	HKEY fileListKey;

	DWORD exists;
	DWORD dataType;
	DWORD data;
	DWORD size;
			
	char filename[256];
	char valueName[10];

	int i;

	RegCreateKeyEx(HKEY_CURRENT_USER,"software",0,"",REG_OPTION_NON_VOLATILE,KEY_READ,NULL,&softwareKey,&exists);		
	RegCreateKeyEx(softwareKey,"Terii senshi",0,"",REG_OPTION_NON_VOLATILE,KEY_READ,NULL,&companyKey,&exists);
	RegCreateKeyEx(companyKey,"ff3se",0,"",REG_OPTION_NON_VOLATILE,KEY_READ,NULL,&productKey,&exists);
	RegCreateKeyEx(productKey,"Settings",0,"",REG_OPTION_NON_VOLATILE,KEY_READ,NULL,&settingsKey,&exists);
	RegCreateKeyEx(productKey,"Recent File List",0,"",REG_OPTION_NON_VOLATILE,KEY_READ,NULL,&fileListKey,&exists);
	
	size = 10;

	//Read showGrid
	if(RegQueryValueEx(settingsKey,"grid",NULL,&dataType,(BYTE *)&data,&size) == ERROR_SUCCESS)
		showGrid = (BYTE)data;
	else
		showGrid = 1;
	
	//Read gridMinMag
	if(RegQueryValueEx(settingsKey,"gridMinMag",NULL,&dataType,(BYTE *)&data,&size) == ERROR_SUCCESS)
		gridMinMag = (BYTE)data;
	else
		gridMinMag = 5;

	//Set up background image
	if(RegQueryValueEx(settingsKey,"background",NULL,&dataType,(BYTE *)&data,&size) == ERROR_SUCCESS)
		backImage = (BYTE)data;
	else
		backImage = 0;

	//Read window's X and Y position
	if(RegQueryValueEx(settingsKey,"xPos",NULL,&dataType,(BYTE *)&data,&size) == ERROR_SUCCESS)
		xPos = data;
	else
		xPos = 50;

	if(RegQueryValueEx(settingsKey,"yPos",NULL,&dataType,(BYTE *)&data,&size) == ERROR_SUCCESS)
		yPos = data;
	else
		yPos = 50;

	//Read which tab is selected
	if(RegQueryValueEx(settingsKey,"tab",NULL,&dataType,(BYTE *)&data,&size) == ERROR_SUCCESS)
		tabSelected = (BYTE)data;
	else
		tabSelected = 0;

	//Read magnifaction levels for tabs
	if(RegQueryValueEx(settingsKey,"tab1Mag",NULL,&dataType,(BYTE *)&data,&size) == ERROR_SUCCESS)
		tab1Mag = (BYTE)data;
	else
		tab1Mag = 4;
	if(RegQueryValueEx(settingsKey,"tab3Mag",NULL,&dataType,(BYTE *)&data,&size) == ERROR_SUCCESS)
		tab3Mag = (BYTE)data;
	else
		tab3Mag = 6;
	if(RegQueryValueEx(settingsKey,"tab4Mag",NULL,&dataType,(BYTE *)&data,&size) == ERROR_SUCCESS)
		tab4Mag = (BYTE)data;
	else
		tab4Mag = 10;

	//Read recent file list
	numRecentFiles = 0;

	size = 256;
	
	for(i = 0;i < 4;i++)
		{
		sprintf(valueName,"file%d",i + 1);
		if(RegQueryValueEx(fileListKey,valueName,NULL,&dataType,(BYTE *)filename,&size) == ERROR_SUCCESS)
			{
			strcpy(recentFile[numRecentFiles++],filename);
			}
		}

	RegCloseKey(softwareKey);
	RegCloseKey(companyKey);
	RegCloseKey(productKey);
	RegCloseKey(settingsKey);
	RegCloseKey(fileListKey);
	}

/*************************************/
//Saves the configuration data to file
void Options::saveConfig(void)
	{
	HKEY softwareKey;
	HKEY companyKey;
	HKEY productKey;
	HKEY settingsKey;
	HKEY fileListKey;

	DWORD exists;
	DWORD data;

	RECT r;

	char valueName[10];
	
	int i;

	RegCreateKeyEx(HKEY_CURRENT_USER,"software",0,"",REG_OPTION_NON_VOLATILE,KEY_READ,NULL,&softwareKey,&exists);		
	RegCreateKeyEx(softwareKey,"Terii senshi",0,"",REG_OPTION_NON_VOLATILE,KEY_READ,NULL,&companyKey,&exists);
	RegCreateKeyEx(companyKey,"ff3se",0,"",REG_OPTION_NON_VOLATILE,KEY_READ,NULL,&productKey,&exists);
	RegCreateKeyEx(productKey,"Settings",0,"",REG_OPTION_NON_VOLATILE,KEY_WRITE,NULL,&settingsKey,&exists);
	RegCreateKeyEx(productKey,"Recent File List",0,"",REG_OPTION_NON_VOLATILE,KEY_WRITE,NULL,&fileListKey,&exists);

	//Save grid settings
	data = showGrid;
	RegSetValueEx(settingsKey,"grid",0,REG_DWORD,(BYTE *)&data,4);
	data = gridMinMag;
	RegSetValueEx(settingsKey,"gridMinMag",0,REG_DWORD,(BYTE *)&data,4);

	//Save background settings
	data = backImage;
	RegSetValueEx(settingsKey,"background",0,REG_DWORD,(BYTE *)&data,4);

	//Save window position
	GetWindowRect(hWnd,&r);

	data = r.left;
	RegSetValueEx(settingsKey,"xPos",0,REG_DWORD,(BYTE *)&data,4);
	data = r.top;
	RegSetValueEx(settingsKey,"yPos",0,REG_DWORD,(BYTE *)&data,4);

	//Save which tab is selected
	data = tabSelected;
	RegSetValueEx(settingsKey,"tab",0,REG_DWORD,(BYTE *)&data,4);

	//Save magnification levels for tabs
	data = tab1Mag;
	RegSetValueEx(settingsKey,"tab1Mag",0,REG_DWORD,(BYTE *)&data,4);
	data = tab3Mag;
	RegSetValueEx(settingsKey,"tab3Mag",0,REG_DWORD,(BYTE *)&data,4);
	data = tab4Mag;
	RegSetValueEx(settingsKey,"tab4Mag",0,REG_DWORD,(BYTE *)&data,4);

	//Save Recent file list
	for(i = 0;i < numRecentFiles;i++)
		{
		sprintf(valueName,"file%d",i + 1);
		RegSetValueEx(fileListKey,valueName,0,REG_SZ,(BYTE *)recentFile[i],strlen(recentFile[i]));
		}

	RegCloseKey(softwareKey);
	RegCloseKey(companyKey);
	RegCloseKey(productKey);
	RegCloseKey(settingsKey);
	RegCloseKey(fileListKey);
	}

/*************************************/
//Sets the grid settings
//Show sets the value of ShowGrid. min sets the value of gridMinMag
void Options::setGrid(int show,int min)
	{
	showGrid = show;

	if(min > 0)
		gridMinMag = min;
	}
