#include "stdafx.h"

#include "portrait.h"
#include "ff3se.h"
#include "tile.h"

Portrait portrait[19];
RGBTRIPLE portraitPals[19][16];


/*************************************/
//Changes the Portrait's palette and updates the bitmap
void Portrait::changePalette(BYTE p)
	{
	int i;

	palette = p;

	for(i = 0;i < 16;i++)
		{
		pic.setColor(i,portraitPals[palette][i]);
		}
	}

/*************************************/
//Clears the portrait
void Portrait::clear(void)
	{
	int x,y;

	for(y = 0;y < 40;y++)
		for(x = 0;x < 40;x++)
			pic.setPixel(x,y,0);
	}

/*************************************/
//Copies the image and palette of portrait #num
void Portrait::copy(BYTE num)
	{
	int x,y;
	int i;

	for(y = 0;y < 40;y++)
		for(x = 0;x < 40;x++)
			pic.setPixel(x,y,portrait[num].getPixel(x,y));
	
	for(i = 0;i < 16;i++)
		{
		portraitPals[palette][i].rgbtBlue = portraitPals[portrait[num].getPalNum()][i].rgbtBlue;
		portraitPals[palette][i].rgbtGreen = portraitPals[portrait[num].getPalNum()][i].rgbtGreen;
		portraitPals[palette][i].rgbtRed = portraitPals[portrait[num].getPalNum()][i].rgbtRed;
		}
	
	changePalette(palette);			//Update bitmap
	}


/*************************************/
void Portrait::flipImage(int flip)
	{
	pic.flipImage(flip);
	}

/*************************************/
//Loads a Portrait from the data pointed to by ptr
void Portrait::load(BYTE *ptr)
	{
	BYTE tile[64];
	int i;

	memcpy(image,ptr,800);

	pic.createBlank(40,40);
	
	for(i = 0;i < 16;i++)
		{
		pic.setColor(i,portraitPals[palette][i]);
		}

	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,0,0);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,1,0);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,2,0);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,3,0);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,0,2);

	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,1,2);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,2,2);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,3,2);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,4,0);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,4,1);

	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,4,2);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,4,3);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,4,4);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,0,4);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,1,4);
	
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,2,4);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,0,1);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,1,1);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,2,1);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,3,1);

	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,0,3);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,1,3);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,2,3);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,3,3);
	loadTile(ptr,tile);
	ptr += 32;
	pic.copyTile(tile,3,4);
	}

/*************************************/
//Saves the portrait to ROM
void Portrait::save(int handle)
	{
	BYTE *ptr;

	ZeroMemory(image,800);

	ptr = image;

	createTile(pic,0,0,ptr);
	ptr += 32;
	createTile(pic,1,0,ptr);
	ptr += 32;
	createTile(pic,2,0,ptr);
	ptr += 32;
	createTile(pic,3,0,ptr);
	ptr += 32;
	createTile(pic,0,2,ptr);
	ptr += 32;

	createTile(pic,1,2,ptr);
	ptr += 32;
	createTile(pic,2,2,ptr);
	ptr += 32;
	createTile(pic,3,2,ptr);
	ptr += 32;
	createTile(pic,4,0,ptr);
	ptr += 32;
	createTile(pic,4,1,ptr);
	ptr += 32;

	createTile(pic,4,2,ptr);
	ptr += 32;
	createTile(pic,4,3,ptr);
	ptr += 32;
	createTile(pic,4,4,ptr);
	ptr += 32;
	createTile(pic,0,4,ptr);
	ptr += 32;
	createTile(pic,1,4,ptr);
	ptr += 32;

	createTile(pic,2,4,ptr);
	ptr += 32;
	createTile(pic,0,1,ptr);
	ptr += 32;
	createTile(pic,1,1,ptr);
	ptr += 32;
	createTile(pic,2,1,ptr);
	ptr += 32;
	createTile(pic,3,1,ptr);
	ptr += 32;

	createTile(pic,0,3,ptr);
	ptr += 32;
	createTile(pic,1,3,ptr);
	ptr += 32;
	createTile(pic,2,3,ptr);
	ptr += 32;
	createTile(pic,3,3,ptr);
	ptr += 32;
	createTile(pic,3,4,ptr);
	
	write(handle,&image,800);
	}

/*************************************/
//Copies the passed Bitmap into the Portrait's image
//Updates the portrait palettes
void Portrait::setImage(Bitmap& bmp)
	{
	int x,y;
	int i;

	for(y = 0;y < 40;y++)
		for(x = 0;x < 40;x++)
			pic.setPixel(x,y,bmp.getPixel(x,y));
	
	for(i = 0;i < 16;i++)
		{
		pic.setColor(i,bmp.getColor(i));
		portraitPals[palette][i].rgbtBlue = bmp.getColor(i).rgbtBlue;
		portraitPals[palette][i].rgbtGreen = bmp.getColor(i).rgbtGreen;
		portraitPals[palette][i].rgbtRed = bmp.getColor(i).rgbtRed;
		}
	}

/*************************************/
//Makes color # num transparent by swapping it with color 0
void Portrait::setTransparency(BYTE num)
	{
	RGBTRIPLE temp;
	
	pic.swapPal(0,num);

	temp = portraitPals[palette][0];

	portraitPals[palette][0].rgbtBlue = portraitPals[palette][num].rgbtBlue;
	portraitPals[palette][0].rgbtGreen = portraitPals[palette][num].rgbtGreen;
	portraitPals[palette][0].rgbtRed = portraitPals[palette][num].rgbtRed;
	portraitPals[palette][num].rgbtBlue = temp.rgbtBlue;
	portraitPals[palette][num].rgbtGreen = temp.rgbtGreen;
	portraitPals[palette][num].rgbtRed = temp.rgbtRed;
	}

/*************************************/
//Updates the Bitmap with the current palette info
void Portrait::updatePalette(void)
	{
	int i;

	for(i = 0;i < 16;i++)
		{
		pic.setColor(i,portraitPals[palette][i]);
		}
	}

/*************************************/
//Loads all the Portraits from the ROM
bool loadPortraits(char *filename)
	{
	int handle;
	BYTE *temp;
	BYTE *ptr;
	int i;
	char c;

	temp = (BYTE *)malloc(20000);
	
	if(temp == NULL)
		return false;
	
	handle = open(filename,O_RDWR | O_BINARY);

	if(handle == -1)
		{
		return false;
		}
	
	loadPortraitPals(handle);

	lseek(handle,0x37100,SEEK_SET);
	for(i = 0;i < 18;i++)
		{
		read(handle,&c,1);

		portrait[i].setPalette(c);
		}

	read(handle,&c,1);		//Skip extra Terra portrait
	read(handle,&c,1);		//Skip extra Vicks & Wedge portrait
	read(handle,&c,1);		//Read palette number for ghost
	portrait[18].setPalette(c);

	lseek(handle,0x2D1F00,SEEK_SET);
	read(handle,temp,20000);

	ptr = temp;
	
	for(i = 0;i < 19;i++)
		{
		portrait[i].load(ptr);
		ptr += 800;
		}
	
	SAFE_DELETE(temp);

	close(handle);

	return true;
	}

/*************************************/
//Loads all 19 portrait palettes from the ROM
void loadPortraitPals(int handle)
	{
	int	i,j;
	unsigned char c,d;
	BYTE temp[32 * 19];
	BYTE *ptr;

	lseek(handle,0x2D5A60,SEEK_SET);		//Write palette number for each portrait
	read(handle,&temp,32 * 19);

	ptr = temp;
	for(i = 0;i < 19;i++)					//Write palette data
		{
		for(j = 0;j < 16;j++)
			{
			c = *(ptr++);
			d = *(ptr++);
			portraitPals[i][j] = getRGBTRIPLE(c,d);
			}	
		}
	}

/*************************************/
//Saves all 19 portrait palettes to ROM
void savePortraitPals(int handle)
	{
	int	i,j;
	BYTE c,d;
	BYTE temp[32 * 19];
	BYTE *ptr;
	RGBTRIPLE color;

	lseek(handle,0x2D5A60,SEEK_SET);

	ptr = temp;
	for(i = 0;i < 19;i++)
		{
		for(j = 0;j < 16;j++)
			{
			color = portraitPals[i][j];
			getIntPal(color,&c,&d);
			*(ptr++) = c;
			*(ptr++) = d;
			}	
		}
	write(handle,&temp,32 * 19);

	lseek(handle,0x37100,SEEK_SET);
	for(i = 0;i < 18;i++)
		{
		c = portrait[i].getPalNum();
		write(handle,&c,1);
		}
	
	c = portrait[18].getPalNum();
	lseek(handle,2,SEEK_CUR);
	write(handle,&c,1);
	}

/*************************************/
//Saves all portraits to ROM
bool savePortraits(char *filename)
	{
	int handle;
	int i;
		
	handle = open(filename,O_RDWR | O_BINARY);

	if(handle == -1)
		{
		MessageBox(NULL,"Error saving portrait data",ff3seTitle,MB_OK | MB_ICONWARNING | MB_TASKMODAL);
		return false;
		}

	savePortraitPals(handle);
	
	lseek(handle,0x2D1F00,SEEK_SET);
	for(i = 0;i < 19;i++)
		{
		portrait[i].save(handle);
		}
	
	close(handle);
	
	return true;
	}

