#include "stdafx.h"

#include "tile.h"
#include "ff3se.h"

/*************************************/
//Creates an 8x8 tile in the ROM format from the Bitmap
//pic is the Bitmap to create a tile from, x and Y are which tile to create
//ptr is the memory location to write the tile to
//numColors is the number of colors in the tile
void createTile(Bitmap& pic,int x,int y,BYTE *ptr,BYTE numColors)
	{
	int x2,y2;
	int x3 = x * 8,y3 = y * 8;
	BYTE val;

	if(numColors == 16)
		ZeroMemory(ptr,32);
	else
		ZeroMemory(ptr,24);
	
	for(y2 = 0;y2 < 8;y2++)
		{
		for(x2 = 0;x2 < 8;x2++)
			{
			val = pic.getPixel(x2 + x3,y2 + y3);
			
			if(val & 0x01)
				ptr[y2 * 2] += BIT(7 - x2);
			if(val & 0x02)
				ptr[y2 * 2 + 1] += BIT(7 - x2);
			if(numColors == 8)
				{
				if(val & 0x04)
					ptr[16 + y2] += BIT(7 - x2);
				}
			else
				{
				if(val & 0x04)
					ptr[16 + y2 * 2] += BIT(7 - x2);
				if(val & 0x08)
					ptr[16 + y2 * 2 + 1] += BIT(7 - x2);
				}
			}
		}
	}


/*************************************/
//Flips the tile along the X-axis
void flipTileHorizontal(BYTE *tile)
	{
	BYTE temp[64];
	int x,y;
	BYTE *ptr;

	memcpy(temp,tile,64);

	ptr = tile;
	
	for(y = 0;y < 8;y++)
		{
		for(x = 0;x < 8;x++)
			{
			*(ptr++) = temp[y * 8 + (7 - x)];
			}
		}	
	}

/*************************************/
//Flips the tile along the Y-axis
void flipTileVertical(BYTE *tile)
	{
	BYTE temp[64];
	int x,y;
	BYTE *ptr;

	memcpy(temp,tile,64);

	ptr = tile;
	
	for(y = 0;y < 8;y++)
		{
		for(x = 0;x < 8;x++)
			{
			*(ptr++) = temp[(7 - y) * 8 + x];
			}
		}	
	}

/*************************************/
//Loads an 8x8 tile into the tile pointer
//ptr points to the data which contains the tile in ROM format
//tile gets the tile data
void loadTile(BYTE *ptr,BYTE *tile,BYTE colors)
	{
	int y;
	BYTE c;

	ZeroMemory(tile,64);

	for(y = 0;y < 8;y++)
		{
		c = *(ptr++);

		tile[7 + y * 8] += (c & 0x01) >> 0;
		tile[6 + y * 8] += (c & 0x02) >> 1;
		tile[5 + y * 8] += (c & 0x04) >> 2;
		tile[4 + y * 8] += (c & 0x08) >> 3;
		tile[3 + y * 8] += (c & 0x10) >> 4;
		tile[2 + y * 8] += (c & 0x20) >> 5;
		tile[1 + y * 8] += (c & 0x40) >> 6;
		tile[0 + y * 8] += (c & 0x80) >> 7;
		
		c = *(ptr++);

		tile[7 + y * 8] += (c & 0x01) << 1;
		tile[6 + y * 8] += (c & 0x02) >> 0;
		tile[5 + y * 8] += (c & 0x04) >> 1;
		tile[4 + y * 8] += (c & 0x08) >> 2;
		tile[3 + y * 8] += (c & 0x10) >> 3;
		tile[2 + y * 8] += (c & 0x20) >> 4;
		tile[1 + y * 8] += (c & 0x40) >> 5;
		tile[0 + y * 8] += (c & 0x80) >> 6;
		}
	for(y = 0;y < 8;y++)
		{
		c = *(ptr++);

		tile[7 + y * 8] += (c & 0x01) << 2;
		tile[6 + y * 8] += (c & 0x02) << 1;
		tile[5 + y * 8] += (c & 0x04) >> 0;
		tile[4 + y * 8] += (c & 0x08) >> 1;
		tile[3 + y * 8] += (c & 0x10) >> 2;
		tile[2 + y * 8] += (c & 0x20) >> 3;
		tile[1 + y * 8] += (c & 0x40) >> 4;
		tile[0 + y * 8] += (c & 0x80) >> 5;
		
		if(colors == 16)
			{
			c = *(ptr++);

			tile[7 + y * 8] += (c & 0x01) << 3;
			tile[6 + y * 8] += (c & 0x02) << 2;
			tile[5 + y * 8] += (c & 0x04) << 1;
			tile[4 + y * 8] += (c & 0x08) >> 0;
			tile[3 + y * 8] += (c & 0x10) >> 1;
			tile[2 + y * 8] += (c & 0x20) >> 2;
			tile[1 + y * 8] += (c & 0x40) >> 3;
			tile[0 + y * 8] += (c & 0x80) >> 4;
			}
		}
	}

